Package: anansi
Type: Package
Title: Annotation-Based Analysis of Specific Interactions
Version: 1.3.0
Authors@R: c(
   person("Thomaz", "Bastiaanssen", email = "thomazbastiaanssen@gmail.com", 
       role = c("aut", "cre"), comment = c(ORCID = "0000-0001-6891-734X")), 
   person("Thomas", "Quinn", email = "contacttomquinn@gmail.com", 
       role = c("aut"), comment = c(ORCID = "0000-0003-0286-6329")),
   person("Giulio", "Benedetti", email = "giulio.benedetti@utu.fi", 
       role = c("aut"), comment = c(ORCID = "0000-0002-8732-7692")), 
   person("Tuomas", "Borman", email = "tuomas.v.borman@utu.fi", 
       role = c("aut"), comment = c(ORCID = "0000-0002-8563-8884")), 
   person("Leo", "Lahti", email = "leo.lahti@utu.fi",
       role = c("aut"), comment = c(ORCID = "0000-0001-5537-637X"))    
   )
Description: Studies including both microbiome and metabolomics data are 
    becoming more common. Often, it would be helpful to integrate both datasets 
    in order to see if they corroborate each others patterns. All vs all 
    association is imprecise and likely to yield spurious associations. This
    package takes a knowledge-based approach to constrain association search 
    space, only considering metabolite-function pairs that have been recorded in
    a pathway database. This package also provides a framework to assess 
    differential association. 
Depends: R (>= 4.5.0)
Imports: S7, stats, methods, igraph, Matrix, forcats, S4Vectors,
        SummarizedExperiment, MultiAssayExperiment,
        SingleCellExperiment, TreeSummarizedExperiment, rlang, ggplot2,
        ggforce, patchwork, ggraph, tidygraph
Suggests: BiocStyle, dplyr, tidyr, graph, mia, KEGGREST, testthat (>=
        3.0.0), knitr, rmarkdown
License: GPL-3
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.3.3
Config/testthat/edition: 3
biocViews: Microbiome, Metabolomics, Regression, Pathways, KEGG
URL: https://github.com/thomazbastiaanssen/anansi,
        https://thomazbastiaanssen.github.io/anansi
BugReports: https://github.com/thomazbastiaanssen/anansi/issues
VignetteBuilder: knitr
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/anansi
git_branch: devel
git_last_commit: 536c3f2
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:13:49 UTC; biocbuild
Author: Thomaz Bastiaanssen [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6891-734X>),
  Thomas Quinn [aut] (ORCID: <https://orcid.org/0000-0003-0286-6329>),
  Giulio Benedetti [aut] (ORCID: <https://orcid.org/0000-0002-8732-7692>),
  Tuomas Borman [aut] (ORCID: <https://orcid.org/0000-0002-8563-8884>),
  Leo Lahti [aut] (ORCID: <https://orcid.org/0000-0001-5537-637X>)
Maintainer: Thomaz Bastiaanssen <thomazbastiaanssen@gmail.com>
