# Generated by roxygen2: do not edit by hand

S3method("[<-","anansi::MultiFactor")
S3method("[[<-","anansi::MultiFactor")
S3method(as.data.frame,"anansi::AnansiWeb")
S3method(as.list,"anansi::AnansiWeb")
S3method(as.list,"anansi::MultiFactor")
S3method(levels,"anansi::MultiFactor")
S3method(pairs,"anansi::AnansiWeb")
export("metadata<-")
export(AnansiTale)
export(AnansiWeb)
export(MultiFactor)
export(anansi)
export(asMAE)
export(asMultiFactor)
export(asTSE)
export(dictionary)
export(getEdgeList)
export(getFeaturePairs)
export(getGraph)
export(kegg_link)
export(krebsDemoWeb)
export(linkBiobakeryMap)
export(metadata)
export(pairwiseApply)
export(plotAnansi)
export(randomMultiFactor)
export(randomWeb)
export(tableX)
export(tableY)
export(unfactor)
export(weaveKEGG)
export(weaveWeb)
if (getRversion() < "4.3.0") importFrom("S7", "@")
import(S7)
import(methods)
importClassesFrom(Matrix,Matrix)
importClassesFrom(MultiAssayExperiment,MultiAssayExperiment)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importClassesFrom(TreeSummarizedExperiment,TreeSummarizedExperiment)
importFrom(Matrix,Matrix)
importFrom(Matrix,colSums)
importFrom(Matrix,drop0)
importFrom(Matrix,rowSums)
importFrom(Matrix,rsparsematrix)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,tcrossprod)
importFrom(Matrix,which)
importFrom(MultiAssayExperiment,ExperimentList)
importFrom(MultiAssayExperiment,MultiAssayExperiment)
importFrom(MultiAssayExperiment,experiments)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,isEmpty)
importFrom(S4Vectors,unfactor)
importFrom(S7,new_class)
importFrom(S7,props)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,altExp)
importFrom(SingleCellExperiment,altExpNames)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,colData)
importFrom(TreeSummarizedExperiment,TreeSummarizedExperiment)
importFrom(forcats,lvls_expand)
importFrom(ggforce,facet_col)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_alpha_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggraph,geom_edge_link)
importFrom(ggraph,geom_node_point)
importFrom(ggraph,geom_node_text)
importFrom(ggraph,ggraph)
importFrom(ggraph,scale_edge_colour_gradient2)
importFrom(ggraph,theme_graph)
importFrom(graphics,pairs)
importFrom(igraph,as_graphnel)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,shortest_paths)
importFrom(methods,getClass)
importFrom(methods,is)
importFrom(methods,show)
importFrom(patchwork,plot_layout)
importFrom(patchwork,wrap_plots)
importFrom(rlang,.data)
importFrom(rlang,is_missing)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,lm)
importFrom(stats,model.frame)
importFrom(stats,model.matrix.default)
importFrom(stats,p.adjust)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(stats,terms.formula)
importFrom(stats,update.formula)
importFrom(tidygraph,tbl_graph)
importFrom(utils,data)
importMethodsFrom(Matrix,"%&%")
importMethodsFrom(S4Vectors,"metadata<-")
importMethodsFrom(S4Vectors,droplevels)
importMethodsFrom(S4Vectors,metadata)
importMethodsFrom(S4Vectors,unfactor)
importMethodsFrom(methods,show)
