% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{FMT_metab}
\alias{FMT_metab}
\title{Snippet of the CLR-transformed hippocampal metabolomics data from the FMT
Aging study.}
\format{
A matrix object with three rows, compounds, and 36 columns, samples.
}
\source{
\doi{10.1038/s43587-021-00093-9}
}
\usage{
data("FMT_data", package = "anansi")
}
\description{
Snippet of the CLR-transformed hippocampal metabolomics data from the FMT
Aging study.
}
\keyword{datasets}
