% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{tableY}
\alias{tableY}
\alias{tableY-generic}
\title{Get tableY}
\usage{
tableY(x, ...)
}
\arguments{
\item{x}{input object}

\item{...}{additional arguments}
}
\value{
\code{tableY} slot of x
}
\description{
Get tableY
}
\examples{
x <- randomWeb(10)
tableY(x)
}
