% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AnansiWeb-methods.R
\name{metadata<-}
\alias{metadata<-}
\alias{metadata.set}
\alias{metadata<-,anansi::AnansiWeb-method}
\title{Set metadata.}
\arguments{
\item{x}{input object}

\item{...}{additional arguments}

\item{value}{replacement value, coerced to data.frame}
}
\value{
\code{x} with modified \code{metadata} slot.
}
\description{
Set metadata.
}
\details{
Compatible with S4Vectors generic.
}
\examples{
x <- randomWeb(10)
metadata(x) <- cbind(
    metadata(x),
    new_groups = c("A", "B")
)
}
