% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{krebs}
\alias{krebs}
\title{Simplified snippet of the Krebs cycle}
\format{
A \code{data.frame} of two factors, named \code{"Enzyme"} and \code{"Metabolite"},
which contain these respective components of the krebs cycle.
}
\source{
Generated by hand based on the krebs cycle.
}
\usage{
data("krebs", package = "anansi")
}
\description{
Enzymes and metabolites that make up the Krebs cycle. Features share a row
when they play a role in the same reaction.
Used in vignettes, exported to facilitate user exploration.
}
\seealso{
\code{\link[=krebsDemoWeb]{krebsDemoWeb()}}
}
\keyword{datasets}
