% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AnansiWeb-methods.R
\name{metadata}
\alias{metadata}
\alias{metadata,anansi::AnansiWeb-method}
\title{Get metadata.}
\arguments{
\item{x}{input object}

\item{...}{additional arguments}
}
\value{
\code{metadata} slot of x
}
\description{
Get metadata.
}
\details{
Compatible with S4Vectors generic.
}
\examples{
x <- randomWeb(10)
metadata(x)
}
