% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{getEdgeList}
\alias{getEdgeList}
\title{Get a listof edges}
\usage{
getEdgeList(x, ...)
}
\arguments{
\item{x}{input object}

\item{...}{additional arguments}
}
\value{
a two-column data.frame that lists the content of each entry in the
input MultiFactor
}
\description{
Get a listof edges
}
\examples{
x <- randomMultiFactor(n_features = 10)
getEdgeList(x)
}
