% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{dictionary}
\alias{dictionary}
\alias{dictionary-generic}
\title{Get dictionary}
\usage{
dictionary(x, ...)
}
\arguments{
\item{x}{input object}

\item{...}{additional arguments}
}
\value{
\code{dictionary} slot of x
}
\description{
Get dictionary
}
\examples{
x <- randomWeb(10)
dictionary(x)

}
