% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{FMT_KOs}
\alias{FMT_KOs}
\title{Snippet of the CLR-transformed inferred functional data from the FMT Aging
study.}
\format{
A marix object with 6468 rows, KOs, and 36 columns, samples.
}
\source{
\doi{10.1038/s43587-021-00093-9}
}
\usage{
data("FMT_data", package = "anansi")
}
\description{
Piphillin was used to infer functions from the 16S sequencing
data in terms of KOs. Unfortunately, the Piphillin algorithm is
proprietary and has since been taken down.
}
\keyword{datasets}
