% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{AnansiTale}
\alias{AnansiTale}
\alias{AnansiTale-class}
\title{AnansiTale S7 container class. Not intended for general use.}
\usage{
AnansiTale(
  subject = character(0),
  type = character(0),
  df = integer(0),
  estimates = integer(0),
  f.values = integer(0),
  t.values = integer(0),
  p.values = integer(0)
)
}
\arguments{
\item{subject}{A character that describes the data that was queried.}

\item{type}{A character that describes type of parameter contained in the
\code{estimates} slot. For example r.values for correlations or r.squared
for models.}

\item{df}{a vector of length 2, containing df1 and df2 corresponding to the
F-ratio considered.}

\item{estimates}{A matrix containing the estimates for the parameters named
in the \code{type} slot.}

\item{f.values}{A matrix containing the f-values, for least-squares.}

\item{t.values}{A matrix containing the t-values, for correlations.}

\item{p.values}{A matrix containing the p.values for the parameters named in
the \code{type} slot.}
}
\value{
an \code{AnansiTale}
}
\description{
\code{AnansiTale} is the main container that will hold your
stats output data coming out of the \code{anansi} pipeline.
}
\section{Slots}{

\describe{
\item{\code{subject}}{A character that describes the data that was queried.}

\item{\code{type}}{A character that describes type of parameter contained in the
\code{estimates} slot. For example r.values for correlations or r.squared
for models.}

\item{\code{df}}{a vector of length 2, containing df1 and df2 corresponding to the
F-ratio considered.}

\item{\code{estimates}}{A matrix containing the estimates for the parameters named in
the \code{type} slot.}

\item{\code{f.values}}{A matrix containing the f-values, for least-squares.}

\item{\code{t.values}}{A matrix containing the t-values, for correlations.}

\item{\code{p.values}}{A matrix containing the p.values for the parameters named in
the \code{type} slot.}
}}

\examples{
AnansiTale

}
