% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/methods-getGraph.R
\name{getGraph}
\alias{getGraph}
\alias{getGraph.list}
\title{Get a graph object.}
\usage{
getGraph(x, ...)
}
\arguments{
\item{x}{input}

\item{...}{additional arguments (currently not used).}
}
\value{
a specified graph object.
}
\description{
Get a graph object.
}
\examples{
# Show methods
getGraph
}
\seealso{
\code{\link{MultiFactor-methods}}
}
