% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{getFeaturePairs}
\alias{getFeaturePairs}
\title{Get a list of all pairs of features}
\usage{
getFeaturePairs(x, ...)
}
\arguments{
\item{x}{input object}

\item{...}{additional arguments for specific methods}
}
\value{
an list of two-column data.frames that represent all feature pairs.
}
\description{
Get a list of all pairs of features
}
\examples{
x <- randomWeb(10)
head(getFeaturePairs(x), 3)

}
