% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{FMT_metadata}
\alias{FMT_metadata}
\title{Snippet of the metadata from the FMT Aging study.}
\format{
A data.frame object with 36 rows, samples, and two columns, denoting
sample ID and treatment group, respectively.
}
\source{
\doi{10.1038/s43587-021-00093-9}
}
\usage{
data("FMT_data", package = "anansi")
}
\description{
There were three treatment groups in the study that all received
faecal microbiota transplantation (FMT). Young mice that received FMT
from young mice (Young yFMT), aged mice that received FMT from aged mice
(Aged oFMT) and aged mice that received FMT from young mice (Aged yFMT).
}
\keyword{datasets}
