% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anansiCorTestByGroup.R
\name{anansiCor}
\alias{anansiCor}
\title{Compute r-statistics for each featureY-featureX pair in the dictionary.
Typically, the main \code{anansi()} function will run this for you.}
\usage{
anansiCor(web, group.bool)
}
\arguments{
\item{web}{An \code{AnansiWeb} object, containing two tables with omics data and a
dictionary that links them. See \code{weaveWebFromTables()} for how to weave
a web.}

\item{group.bool}{A boolean vector used to select which samples should be
included in the correlations.}
}
\value{
A matrix of r-statistics.
}
\description{
Compute r-statistics for each featureY-featureX pair in the dictionary.
Typically, the main \code{anansi()} function will run this for you.
}
\seealso{
\code{\link[=anansi]{anansi()}} \cr \code{\link[=anansiCorTestByGroup]{anansiCorTestByGroup()}}
}
\keyword{internal}
